#!/usr/bin/env python3

'''Config module.'''

import os
import sys

from ncmpy.util import printerr

class Config:
    '''Config class.'''

    def __init__(self):
        self.mpd_host = 'localhost'
        self.mpd_port = 6600
        self.enable_rating = True
        self.lyrics_dir = os.path.expanduser('~/.ncmpy/lyrics')

        for filename in (
                os.path.expanduser('~/.config/ncmpy/ncmpy.conf'),
                '/etc/ncmpy/ncmpy.conf',
                ):

            if not os.path.isfile(filename):
                continue

            with open(filename, 'rt') as f:
                line_num = 0
                for l in f:
                    line_num += 1

                    if l.startswith('#'):
                        continue

                    try:
                        o, a = map(str.strip, l.split('=', 1))

                        if o == 'MPD_HOST':
                            self.mpd_host = a
                        elif o == 'MPD_PORT':
                            self.mpd_port = int(a)
                        elif o == 'ENABLE_RATING':
                            self.enable_rating = a in ['True', 'true', 'Yes', 'yes', '1']
                        elif o == 'LYRICS_DIR':
                            self.lyrics_dir = os.path.expanduser(a)
                        else:
                            raise
                    except:
                        printerr('Invalid line {} in config file {}'.format(line_num, filename))
                        sys.exit(1)
            break

# Global configuration.
conf = Config()




#!/usr/bin/env python3

'''Pane module.'''

import curses
import locale
import mpd
import os
import threading
import time

from ncmpy import lrc
from ncmpy import ttplyrics

from ncmpy.config import *
from ncmpy.util import *

class Pane():
    '''Base class of all panes.'''

    def __init__(self, win, main):
        '''
        Initialize a pane.

        win - A curses.window instance.
        main - A Ncmpy instance.
        '''

        self.win = win
        self.main = main
        self.mpc = main.mpc
        self.board = main.board
        self.height, self.width = self.win.getmaxyx()

    def update_data(self):
        '''Update data.'''

        self.status = self.main.status
        self.stats = self.main.stats
        self.currentsong = self.main.currentsong

    def round1(self, c):
        '''Round one.'''

        pass

    def round2(self):
        '''Round two.'''

        pass

    def update_win(self):
        '''Update window.'''

        pass

    def resize_win(self):
        '''Resize window.'''

        pass

class BarPane(Pane):
    '''Bar pane.'''

    def __init__(self, win, main):
        Pane.__init__(self, win, main)

    def bar_resize_win(self, y, x):
        '''Resize bar window.'''

        self.win.resize(1, self.main.width)
        self.height, self.width = self.win.getmaxyx()
        self.win.mvwin(y, x)

class BlockPane(Pane):
    '''Block pane.'''

    def __init__(self, win, main):
        Pane.__init__(self, win, main)

    def block_resize_win(self):
        '''Resize block window.'''

        self.win.resize(self.main.height - 4, self.main.width)
        self.height, self.width = self.win.getmaxyx()
        self.win.mvwin(2, 0)

    def resize_win(self):
        self.block_resize_win()

class ScrollPane(BlockPane):
    '''Scroll pane.'''

    def __init__(self, win, main):
        BlockPane.__init__(self, win, main)

        # Number of lines in total.
        self.num = 0
        # Beginning line.
        self.beg = 0

    def line_down(self):
        if self.beg < self.num - self.height:
            self.beg += 1

    def line_up(self):
        if self.beg > 0:
            self.beg -= 1

    def page_down(self):
        self.beg = min(self.beg + self.height, self.num - self.height)

    def page_up(self):
        self.beg = max(self.beg - self.height, 0)

    def locate(self, pos):
        '''Select pos, and put in the center.'''

        if pos >= self.height // 2:
            self.beg = pos - self.height // 2
        else:
            self.beg = 0

class CursedPane(BlockPane):
    '''Pane with cursor.'''

    def __init__(self, win, main):
        BlockPane.__init__(self, win, main)

        # Number of lines in total.
        self.num = 0
        # Beginning line.
        self.beg = 0
        # Selected line.
        self.sel = 0
        # Current line.
        self.cur = 0

    def line_down(self):
        if self.sel < self.num - 1:
            self.sel += 1
            if self.sel - self.beg == self.height:
                self.beg += 1

    def line_up(self):
        if self.sel > 0:
            self.sel -= 1
            if self.sel - self.beg == -1:
                self.beg -= 1

    def page_down(self):
        if self.sel < self.num - self.height:
            self.sel += self.height
            self.beg = min(self.beg + self.height, self.num - self.height)
        else:
            self.sel = self.num - 1
            self.beg = max(self.num - self.height, 0)

    def page_up(self):
        if self.sel < self.height:
            self.sel = 0
            self.beg = 0
        else:
            self.sel -= self.height
            self.beg = max(self.beg - self.height, 0)

    def select_top(self):
        self.sel = self.beg

    def select_middle(self):
        self.sel = min(self.beg + self.height // 2, self.num - 1)

    def select_bottom(self):
        self.sel = min(self.beg + self.height - 1, self.num - 1)

    def to_first(self):
        self.beg = 0
        self.sel = 0

    def to_last(self):
        self.beg = max(self.num - 1, 0)
        self.sel = max(self.num - 1, 0)

    def locate(self, pos):
        '''Select pos, and put in the center.'''

        if pos >= self.height // 2:
            self.beg = pos - self.height // 2
        else:
            self.beg = 0
        self.sel = pos

    def clamp(self, n):
        '''Clamp value into range [0, num).'''

        return max(min(n, self.num - 1), 0)

    def block_resize_win(self):
        BlockPane.block_resize_win(self)
        self.sel = min(self.sel, self.beg + self.height - 1)

    def search(self, panename, c):
        '''Search in panes.'''

        if panename == 'Queue':
            items = self.queue
        elif panename in ['Database', 'Artist-Album', 'Search']:
            items = self.items

        if self.main.search and self.main.search_di:
            di = {
                    ord('/') : 1,
                    ord('?') : -1,
                    ord('n') : self.main.search_di,
                    ord('N') : -self.main.search_di
                    }[c]
            has_match = False

            for i in [k % len(items) \
                    for k in range(self.sel + di, self.sel + di + di * len(items), di)]:
                item = items[i]

                if panename in ['Queue', 'Search']:
                    title = get_tag('title', item) or os.path.basename(item['file'])
                elif panename == 'Database':
                    title = list(item.values())[0]
                elif panename == 'Artist-Album':
                    if self._type in ['artist', 'album']:
                        title = item
                    elif self._type == 'song':
                        title = get_tag('title', item) or os.path.basename(item['file'])

                if title.find(self.main.search) != -1:
                    has_match = True
                    if di == 1 and i <= self.sel:
                        self.board['msg'] = 'search hit BOTTOM, continuing at TOP'
                    elif di == -1 and i >= self.sel:
                        self.board['msg'] = 'search hit TOP, continuing at BOTTOM'
                    self.locate(i)
                    break

            if not has_match:
                self.board['msg'] = 'Pattern not found: {}'.format(self.main.search)

class MenuPane(BarPane):
    '''Display pane name, play mode and volume.'''

    BLANK = ' ' * 5

    def __init__(self, win, main):
        BarPane.__init__(self, win, main)
        self.win.attron(curses.A_BOLD)

    def build_menu_str(self):
        title_str = self.main.tpanename
        mode_str = ('[con]' if int(self.status['consume']) else self.BLANK) + \
                ('[ran]' if int(self.status['random']) else self.BLANK) + \
                ('[rep]' if int(self.status['repeat']) else self.BLANK) + \
                ('[sin]' if int(self.status['single']) else self.BLANK)
        vol_str = 'Volume: ' + self.status['volume'] + '%'

        state_str = '{}    {}'.format(mode_str, vol_str)
        title_len = self.width - len(state_str)
        menu_str = title_str.ljust(title_len) + state_str

        return menu_str

    def update_win(self):
        menu_str = self.build_menu_str()

        # Must use insstr instead of addstr, because addstr cannot draw the last character (will
        # raise an exception). The same applies to other panes.
        self.win.erase()
        self.win.insstr(0, 0, menu_str)
        self.win.noutrefresh()

    def resize_win(self):
        self.bar_resize_win(0, 0)

class LinePane(BarPane):
    '''Horizontal line.'''

    def __init__(self, win, main):
        BarPane.__init__(self, win, main)

    def update_win(self):
        self.win.erase()
        self.win.insstr(0, 0, self.width * '-')
        self.win.noutrefresh()

    def resize_win(self):
        self.bar_resize_win(1, 0)

class ProgressPane(BarPane):
    '''Show playing progress.'''

    def __init__(self, win, main):
        BarPane.__init__(self, win, main)

    def build_prog_str(self):
        '''Build progress str.'''

        state = self.status.get('state')
        if state == 'stop':
            return '-' * self.width
        else:
            tm = self.status.get('time')
            elapsed, total = tm.split(':')
            pos = int((float(elapsed) / float(total)) * (self.width - 1))
            return '=' * pos + '0' + '-' * (self.width - pos - 1)

    def update_win(self):
        prog_str = self.build_prog_str()

        self.win.erase()
        self.win.insstr(0, 0, prog_str)
        self.win.noutrefresh()

    def resize_win(self):
        self.bar_resize_win(self.main.height - 2, 0)

class StatusPane(BarPane):
    '''Show playing status, elapsed/total time.'''

    state_name = {
            'play' : 'Playing',
            'stop' : 'Stopped',
            'pause' : 'Paused',
            }

    def __init__(self, win, main):
        BarPane.__init__(self, win, main)
        self.win.attron(curses.A_BOLD)

    def build_title_str(self):
        '''Build title str.'''

        state = self.status.get('state')
        song = self.currentsong
        title = song and (song.get('title') or os.path.basename(song.get('file'))) or ''
        return '{} > {}'.format(self.state_name[state], title)

    def build_tm_str(self):
        '''Build tm str.'''

        tm = self.status.get('time') or '0:0'
        elapsed, total = map(int, tm.split(':'))
        elapsed_mm, elapsed_ss, total_mm, total_ss = \
                elapsed // 60, elapsed % 60, total // 60, total % 60
        return '[{0}:{1:02d} ~ {2}:{3:02d}]'.format(elapsed_mm, elapsed_ss, total_mm, total_ss)

    def update_win(self):
        # Use two strs because it's difficult to calculate display length of unicode characters.
        title_str = self.build_title_str()
        tm_str = self.build_tm_str()

        self.win.erase()
        self.win.insstr(0, 0, title_str)
        self.win.insstr(0, self.width - len(tm_str), tm_str)
        self.win.noutrefresh()

    def resize_win(self):
        self.bar_resize_win(self.main.height - 1, 0)

class MessagePane(BarPane):
    '''Show message and get user input.'''

    def __init__(self, win, main):
        BarPane.__init__(self, win, main)
        self.msg = None
        self.timeout = 0

    def getstr(self, prompt):
        '''Get user input with prompt <prompt>.'''

        curses.nocbreak()
        curses.echo()
        curses.curs_set(1)
        self.win.move(0, 0)
        self.win.clrtoeol()
        self.win.addstr('{}: '.format(prompt), curses.A_BOLD)
        s = self.win.getstr(0, len(prompt) + 2)
        curses.curs_set(0)
        curses.noecho()
        curses.cbreak()
        return s.decode()

    def update_win(self):
        msg = self.board.get('msg')
        if msg:
            self.msg = msg
            self.timeout = 10   # Magic!

        # TODO. Use a real timer.
        if self.timeout > 0:
            self.win.erase()
            self.win.insstr(0, 0, self.msg, curses.A_BOLD)
            self.win.noutrefresh()
            self.timeout -= 1

    def resize_win(self):
        self.bar_resize_win(self.main.height - 1, 0)

class HelpPane(ScrollPane):
    '''Help.'''

    def __init__(self, win, main):
        ScrollPane.__init__(self, win, main)
        self.lines = [
                ('group', 'Global', None),
                ('hline', None, None),
                ('item', 'F1', 'Help'),
                ('item', 'F2', 'Queue'),
                ('item', 'F3', 'Database'),
                ('item', 'F4', 'Lyrics'),
                ('item', 'F5', 'Artist-Album'),
                ('item', 'F6', 'Search'),
                ('item', 'F7', 'Info'),
                ('item', 'F8', 'Output'),
                ('blank', None, None),
                ('item', 'q', 'quit'),
                ('blank', None, None),

                ('group', 'Playback', None),
                ('hline', None, None),
                ('item', 'Space', 'Play/Pause'),
                ('item', 's', 'Stop'),
                ('item', '>', 'next song'),
                ('item', '<', 'previous song'),
                ('blank', None, None),
                ('item', 'u', 'consume mode'),
                ('item', 'i', 'random mode'),
                ('item', 'o', 'repeat mode'),
                ('item', 'p', 'single mode'),
                ('blank', None, None),
                ('item', '9', 'volume down'),
                ('item', '0', 'volume up'),
                ('blank', None, None),
                ('item', 'left', 'seek +1'),
                ('item', 'right', 'seek -1'),
                ('item', 'down', 'seek -1%'),
                ('item', 'up', 'seek +1%'),
                ('blank', None, None),

                ('group', 'Movement', None),
                ('hline', None, None),
                ('item', 'j', 'go one line down'),
                ('item', 'k', 'go one line up'),
                ('item', 'f', 'go one page down'),
                ('item', 'b', 'go one page up'),
                ('item', 'g', 'go to top of list'),
                ('item', 'G', 'go to bottom of list'),
                ('item', 'H', 'go to top of screen'),
                ('item', 'M', 'go to middle of screen'),
                ('item', 'L', 'go to bottom of screen'),
                ('blank', None, None),
                ('item', '/', 'search down'),
                ('item', '?', 'search up'),
                ('item', 'n', 'next match'),
                ('item', 'N', 'previous match'),
                ('blank', None, None),

                ('group', 'Queue', ''),
                ('hline', None, None),
                ('item', 'Enter', 'Play'),
                ('item', 'l', 'select and center current song'),
                ('item', '\'', 'toggle auto center'),
                ('item', ';', 'locate selected song in database'),
                ('blank', None, None),
                ('item', 'x', 'remove song rating'),
                ('item', '1', 'rate selected song as     *'),
                ('item', '2', 'rate selected song as    **'),
                ('item', '3', 'rate selected song as   ***'),
                ('item', '4', 'rate selected song as  ****'),
                ('item', '5', 'rate selected song as *****'),
                ('blank', None, None),
                ('item', 'J', 'Move down selected song'),
                ('item', 'K', 'Move up selected song'),
                ('item', 'e', 'shuffle queue'),
                ('item', 'c', 'clear queue'),
                ('item', 'a', 'add all songs from database'),
                ('item', 'd', 'delete selected song from queue'),
                ('item', 'S', 'save queue to playlist'),
                ('item', 'O', 'load queue from playlist'),
                ('blank', None, None),

                ('group', 'Database', ''),
                ('hline', None, None),
                ('item', 'Enter', 'open directory / play song / load playlist'),
                ('item', '\'', 'go to parent directory'),
                ('item', '"', 'go to root directory'),
                ('item', 'a', 'append song to queue recursively'),
                ('item', ';', 'locate selected song in queue'),
                ('item', 'U', 'update database'),
                ('blank', None, None),

                ('group', 'Lyrics', ''),
                ('hline', None, None),
                ('item', 'l', 'center current line'),
                ('item', '\'', 'toggle auto center'),
                ('item', 'K', 'save lyrics'),
                ('blank', None, None),

                ('group', 'Artist-Album', ''),
                ('hline', None, None),
                ('item', 'Enter', 'open level / play song'),
                ('item', '\'', 'go to parent level'),
                ('item', '"', 'go to root level'),
                ('item', 'a', 'append song to queue recursively'),
                ('item', ';', 'locate selected song in queue'),
                ('blank', None, None),

                ('group', 'Search', ''),
                ('hline', None, None),
                ('item', 'B', 'start a database search, syntax = <tag_name>:<tag_value>'),
                ('item', 'Enter', 'play song'),
                ('item', 'a', 'append song to queue'),
                ('item', ';', 'locate selected song in queue'),
                ('blank', None, None),

                ('group', 'Info', ''),
                ('hline', None, None),
                ('blank', None, None),

                ('group', 'Output', ''),
                ('hline', None, None),
                ('item', 't', 'toggle output'),
                ('blank', None, None),
                ]
        self.num = len(self.lines)

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()

    def update_win(self):
        self.win.erase()
        for i in range(self.beg, min(self.beg + self.height, self.num)):
            line = self.lines[i]
            if line[0] == 'group':
                self.win.insstr(i - self.beg, 6, line[1], curses.A_BOLD)
            elif line[0] == 'hline':
                self.win.attron(curses.A_BOLD)
                self.win.hline(i - self.beg, 3, '-', self.width - 6)
                self.win.attroff(curses.A_BOLD)
            elif line[0] == 'item':
                self.win.insstr(i - self.beg, 0, line[1].rjust(20) + ' : ' + line[2])
            elif line[0] == 'blank':
                pass
        self.win.noutrefresh()

class QueuePane(CursedPane):
    '''Queue = current playlist.'''

    def __init__(self, win, main):
        CursedPane.__init__(self, win, main)

        # Playlist version.
        self.pl_version = -1

        # Auto center current song.
        self.auto_center = False

    def update_data(self):
        CursedPane.update_data(self)

        # Fetch playlist if version is different.
        if self.pl_version != int(self.status['playlist']):
            self.queue = self.mpc.playlistinfo()
            self.num = len(self.queue)
            self.beg = self.clamp(self.beg)
            self.sel = self.clamp(self.sel)

            for song in self.queue:
                if conf.enable_rating:
                    try:
                        rating = int(self.mpc.sticker_get(\
                                'song', song['file'], 'rating').split('=',1)[1])
                    except mpd.CommandError:
                        rating = 0
                    finally:
                        song['rating'] = rating
                else:
                    song['rating'] = 0

            self.pl_version = int(self.status['playlist'])

        self.cur = ('song' in self.status) and int(self.status['song']) or 0

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()
        elif c == ord('H'):
            self.select_top()
        elif c == ord('M'):
            self.select_middle()
        elif c == ord('L'):
            self.select_bottom()
        elif c == ord('g'):
            self.to_first()
        elif c == ord('G'):
            self.to_last()
        elif c == ord('l'):
            self.locate(self.cur)
        elif c == ord('a'):
            self.mpc.add('')
        elif c == ord('c'):
            self.mpc.clear()
            self.num = self.beg = self.sel = self.cur = 0
        elif c == ord('d'):
            if self.num > 0:
                self.main.pending.append('deleteid({})'.format(self.queue[self.sel]['id']))
                self.queue.pop(self.sel)
                if self.sel < self.cur:
                    self.cur -= 1
                self.num -= 1
                self.beg = self.clamp(self.beg)
                self.sel = self.clamp(self.sel)
                self.cur = self.clamp(self.cur)
        elif c == ord('J'):
            if self.sel + 1 < self.num:
                self.main.pending.append('swap({}, {})'.format(self.sel, self.sel + 1))
                self.queue[self.sel], self.queue[self.sel + 1] = \
                        self.queue[self.sel + 1], self.queue[self.sel]
                if self.cur == self.sel:
                    self.cur += 1
                elif self.cur == self.sel + 1:
                    self.cur -= 1
                self.line_down()
        elif c == ord('K'):
            if self.sel > 0:
                self.main.pending.append('swap({}, {})'.format(self.sel, self.sel - 1))
                self.queue[self.sel - 1], self.queue[self.sel] = \
                        self.queue[self.sel], self.queue[self.sel - 1]
                if self.cur == self.sel - 1:
                    self.cur += 1
                elif self.cur == self.sel:
                    self.cur -= 1
                self.line_up()
        elif c == ord('e'):
            self.mpc.shuffle()
        elif c == ord('\n'):
            self.mpc.playid(self.queue[self.sel]['id'])
        elif c in range(ord('1'), ord('5') + 1):
            if conf.enable_rating:
                rating = c - ord('0')
                if 0 <= self.cur and self.cur < len(self.queue):
                    song = self.queue[self.cur]
                    self.mpc.sticker_set('song', song['file'], 'rating', rating)
                    song['rating'] = rating
        elif c == ord('x'):
            if conf.enable_rating:
                if 0 <= self.cur and self.cur < len(self.queue):
                    song = self.queue[self.cur]
                    try:
                        self.mpc.sticker_delete('song', song['file'], 'rating')
                    except mpd.CommandError as e:
                        self.board['msg'] = str(e)
                    else:
                        song['rating'] = 0
        elif c in [ord('/'), ord('?'), ord('n'), ord('N')]:
            self.search('Queue', c)
        elif c == ord('\''):
            self.auto_center = not self.auto_center
        elif c == ord(';'):
            self.board['database-locate'] = self.queue[self.sel]['file']

        # Record selected song in board.
        if self.num > 0:
            self.board['queue-selected'] = self.queue[self.sel]

    def round2(self):
        uri = self.board.get('queue-locate')
        if uri:
            for i in range(len(self.queue)):
                if uri == self.queue[i]['file']:
                    self.locate(i)
                    break
            else:
                self.board['msg'] = 'Not found in playlist'

        # Auto center.
        if self.auto_center:
            self.locate(self.cur)

    def update_win(self):
        self.win.erase()
        for i in range(self.beg, min(self.beg + self.height, self.num)):
            item = self.queue[i]
            title = get_tag('title', item) or os.path.basename(item['file'])
            rating = item['rating']
            tm = format_time(item['time'])

            if i == self.cur:
                self.win.attron(curses.A_BOLD)
            if i == self.sel:
                self.win.attron(curses.A_REVERSE)
            self.win.hline(i - self.beg, 0, ' ', self.width)
            self.win.addnstr(i - self.beg, 0, title, self.width - 18)
            self.win.addnstr(i - self.beg, self.width - 16, rating * '*', 5)
            self.win.insstr(i - self.beg, self.width - len(tm), tm)
            if i == self.sel:
                self.win.attroff(curses.A_REVERSE)
            if i == self.cur:
                self.win.attroff(curses.A_BOLD)
        self.win.noutrefresh()

class DatabasePane(CursedPane):
    '''All songs/directories/playlists in database.'''

    def __init__(self, win, main):
        CursedPane.__init__(self, win, main)

        # Current dir.
        self.dir = ''
        self.items = self.list_items()

    def list_items(self, keeppos=False):
        '''
        List contents of current dir.

        This method is called when current dir changes (e.g. database update), or new items are
        added or removed (e.g. playlist add/delete).
        '''

        items = self.mpc.lsinfo(self.dir)
        items.insert(0, {'directory' : '..'})
        self.num = len(items)
        if keeppos:
            self.beg = self.clamp(self.beg)
            self.sel = self.clamp(self.sel)
        else:
            self.beg = 0
            self.sel = 0
        return items

    def update_data(self):
        CursedPane.update_data(self)

        if 'database-updated' in self.board:
            self.dir = ''
            self.items = self.list_items()
            self.board['msg'] = 'Database updated.'

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()
        elif c == ord('H'):
            self.select_top()
        elif c == ord('M'):
            self.select_middle()
        elif c == ord('L'):
            self.select_bottom()
        elif c == ord('g'):
            self.to_first()
        elif c == ord('G'):
            self.to_last()
        elif c == ord('\''):
            oldroot = self.dir
            self.dir = os.path.dirname(self.dir)
            self.items = self.list_items()
            for i in range(len(self.items)):
                if self.items[i].get('directory') == oldroot:
                    self.locate(i)
                    break
        elif c == ord('"'):
            self.dir = ''
            self.items = self.list_items()
        elif c == ord('\n'):
            item = self.items[self.sel]
            if ('directory' in item):
                uri = item['directory']
                if uri == '..':
                    oldroot = self.dir
                    self.dir = os.path.dirname(self.dir)
                    self.items = self.list_items()
                    for i in range(len(self.items)):
                        if self.items[i].get('directory') == oldroot:
                            self.locate(i)
                            break
                else:
                    self.dir = uri
                    self.items = self.list_items()

            elif ('file' in item):
                uri = item['file']
                songs = self.mpc.playlistfind('file', uri)
                if songs:
                    self.mpc.playid(songs[0]['id'])
                else:
                    self.mpc.add(uri)
                    song = self.mpc.playlistfind('file', uri)[0]
                    self.mpc.playid(song['id'])
            elif ('playlist' in item):
                name = item['playlist']
                try:
                    self.mpc.load(name)
                except mpd.CommandError as e:
                    self.board['msg'] = str(e).rsplit('} ')[1]
                else:
                    self.board['msg'] = 'Playlist {} loaded'.format(name)
        elif c == ord('a'):
            item = self.items[self.sel]
            if ('directory' in item):
                uri = item['directory']
            else:
                uri = item['file']
            if uri == '..':
                self.mpc.add(os.path.dirname(self.dir))
            else:
                self.mpc.add(uri)
        elif c == ord('d'):
            item = self.items[self.sel]
            if ('playlist' in item):
                name = item['playlist']
                try:
                    self.mpc.rm(name)
                except mpd.CommandError as e:
                    self.board['msg'] = str(e).rsplit('} ')[1]
                else:
                    self.board['msg'] = 'Playlist {} deleted'.format(name)
                    self.items = self.list_items(keeppos=True)
        elif c == ord('U'):
            self.mpc.update()
        elif c in [ord('/'), ord('?'), ord('n'), ord('N')]:
            self.search('Database', c)
        elif c == ord(';'):
            # tell QUEUE we want to locate a song
            item = self.items[self.sel]
            if ('file' in item):
                self.board['queue-locate'] = item.get('file')
            else:
                self.board['msg'] = 'No song selected'

        # Record selected song in borard.
        self.board['database-selected'] = self.items[self.sel].get('file')

    def round2(self):
        # if there's a path request, rebuild view, using
        # dirname(path) as display dir, and search for the
        # requested song.
        uri = self.board.get('database-locate')
        if uri:
            self.dir = os.path.dirname(uri)
            self.items = self.list_items()
            for i in range(len(self.items)):
                if self.items[i].get('file') == uri:
                    self.locate(i)
                    break
            else:
                self.board['msg'] = 'Not found in database'

        # if a playlist is saved, rebuild view, keep original positions
        if self.board.get('playlist') == 'saved':
            self.items = self.list_items(keeppos=True)

    def update_win(self):
        self.win.erase()
        for i in range(self.beg, min(self.beg + self.height, self.num)):
            item = self.items[i]
            if ('directory' in item):
                t, uri = 'directory', item['directory']
            elif ('file' in item):
                t, uri = 'file', item['file']
            elif ('playlist' in item):
                t, uri = 'playlist', item['playlist']

            if i == self.sel:
                self.win.attron(curses.A_REVERSE)
            if t == 'directory':
                self.win.attron(curses.color_pair(1) | curses.A_BOLD)
            elif t == 'playlist':
                self.win.attron(curses.color_pair(2) | curses.A_BOLD)
            self.win.hline(i - self.beg, 0, ' ', self.width)
            self.win.insstr(i - self.beg, 0, os.path.basename(uri))
            if t == 'directory':
                self.win.attroff(curses.color_pair(1) | curses.A_BOLD)
            elif t == 'playlist':
                self.win.attroff(curses.color_pair(2) | curses.A_BOLD)
            if i == self.sel:
                self.win.attroff(curses.A_REVERSE)
        self.win.noutrefresh()

class LyricsPane(ScrollPane, threading.Thread):
    '''Display lyrics.'''

    def __init__(self, win, main):
        ScrollPane.__init__(self, win, main)
        threading.Thread.__init__(self, name='lyrics')

        # directory to save lyrics.
        # Make sure have write permission.
        self._lyrics_dir = conf.lyrics_dir

        # new song, maintained by pane
        self._nsong = None
        # old song, maintained by worker
        self._osong = None
        # title of lyrics to fetch
        self._title = None
        # artist of lyrics to fetch
        self._artist = None
        # current lyrics, oneline str
        self._lyrics = '[00:00.00]Cannot fetch lyrics (No artist/title).'
        # current lyrics timestamp as lists, used by main thread only
        self._ltimes = []
        # current lyrics text as lists, used by main thread only
        self._ltexts = []
        # incicate lyrics state: 'local', 'net', 'saved' or False
        self._lyrics_state = False
        # condition variable for lyrics fetching and display
        self._cv = threading.Condition()

        # auto-center
        self.auto_center = True

    def _transtag(self, tag):
        '''Transform tag into format used by lrc engine.'''

        if tag is None:
            return None
        else:
            return tag.replace(' ', '').lower()

    def update_data(self):
        ScrollPane.update_data(self)

        song = self.currentsong

        # Do nothing if cannot acquire lock.
        if self._cv.acquire(blocking=False):
            self._nsong = song.get('file')
            # If currengsong changes, wake up worker.
            if self._nsong != self._osong:
                self._artist = song.get('artist')
                self._title = song.get('title')
                self._cv.notify()
            self._cv.release()

    def _save_lyrics(self):
        if self._artist and self._title and self._cv.acquire(blocking=False):
            with open(os.path.join(self._lyrics_dir, self._artist.replace('/', '_') + \
                    '-' + self._title.replace('/', '_') + '.lrc'), 'wt') as f:
                f.write(self._lyrics)
            self.board['msg'] = 'Lyrics {}-{}.lrc saved.'.format(self._artist, self._title)
            self._lyrics_state = 'saved'
            self._cv.release()
        else:
            self.board['msg'] = 'Lyrics saving failed.'

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()
        elif c == ord('l'):
            self.locate(self.cur)
        elif c == ord('\''):
            self.auto_center = not self.auto_center
        elif c == ord('K'):
            self._save_lyrics()

    def _parse_lrc(self, lyrics):
        '''Parse lrc lyrics into ltimes and ltexts.'''

        tags, tms = lrc.parse(lyrics)
        sorted_keys = sorted(tms.keys())
        ltimes = [int(i) for i in sorted_keys]
        ltexts = [tms.get(i) for i in sorted_keys]
        return ltimes, ltexts

    def current_line(self):
        '''Calculate line number of current progress.'''

        cur = 0
        tm = self.status.get('time')
        if tm:
            elapsed = int(tm.split(':')[0])
            while cur < self.num and self._ltimes[cur] <= elapsed:
                cur += 1
            cur -= 1
        return cur

    def round2(self):
        # output 'Updating...' if cannot acquire lock
        if self._cv.acquire(blocking=0):
            # if worker reports lyrics fetched
            if self._lyrics_state in ['local', 'net']:
                # parse lrc (and copy lrc from shared mem to non-shared mem)
                self._ltimes, self._ltexts = self._parse_lrc(self._lyrics)
                self.num, self.beg = len(self._ltimes), 0

                # auto-save lyrics
                if self._lyrics_state == 'net' and self.num > 10:
                    self._save_lyrics()
                else:
                    self._lyrics_state = 'saved'

            self._cv.release()
        else:
            self._ltimes, self._ltexts = [0], ['Updating...']
            # set self.num and self.beg
            self.num, self.beg = 1, 0

        # set self.cur, the highlighted line
        self.cur = self.current_line()

        # auto center
        if self.auto_center:
            self.locate(self.cur)

    def update_win(self):
        self.win.erase()
        attr = curses.A_BOLD | curses.color_pair(3)
        for i in range(self.beg, min(self.beg + self.height, self.num)):
            if i == self.cur:
                self.win.insstr(i - self.beg, 0, self._ltexts[i], attr)
            else:
                self.win.insstr(i - self.beg, 0, self._ltexts[i])
        self.win.noutrefresh()

    def run(self):
        self._cv.acquire()
        while True:
            # wait if currentsong doesn't change
            while self._nsong == self._osong:
                self._cv.wait()

            self._lyrics = '[00:00.00]Cannot fetch lyrics (No artist/title).'
            self._lyrics_state = 'local'

            # fetch lyrics if required information is provided
            if self._artist and self._title:
                # try to fetch from local lrc
                lyrics_file = os.path.join(self._lyrics_dir, self._artist.replace('/', '_') + \
                        '-' + self._title.replace('/', '_') + '.lrc')
                if os.path.isfile(lyrics_file):
                    with open(lyrics_file, 'rt') as f:
                        self._lyrics = f.read()
                    # inform round2: lyrics has been fetched
                    self._lyrics_state = 'local'
                # if local lrc doesn't exist, fetch from Internet
                else:
                    self._lyrics = ttplyrics.fetch_lyrics(self._transtag(self._artist), \
                            self._transtag(self._title))
                    # inform round2: lyrics has been fetched
                    self._lyrics_state = 'net'
            self._osong = self._nsong

class InfoPane(ScrollPane):
    '''Information about songs:

        currently playing
        currently selected in queue
        currently selected in database'''

    def __init__(self, win, main):
        ScrollPane.__init__(self, win, main)
        # current playing
        self._cp = {}
        # selected in queue
        self._siq = {}
        # selected in database
        self._sid = {}
        # database.sel's uri cache
        self._dburi = None
        self.lines = [
                ('group', 'Currently Playing', None),
                ('hline', None, None),
                ('item', 'Title', ''),
                ('item', 'Artist', ''),
                ('item', 'Album', ''),
                ('item', 'Track', ''),
                ('item', 'Genre', ''),
                ('item', 'Date', ''),
                ('item', 'Time', ''),
                ('item', 'File', ''),
                ('blank', None, None),

                ('group', 'Currently Selected in Queue', None),
                ('hline', None, None),
                ('item', 'Title', ''),
                ('item', 'Artist', ''),
                ('item', 'Album', ''),
                ('item', 'Track', ''),
                ('item', 'Genre', ''),
                ('item', 'Date', ''),
                ('item', 'Time', ''),
                ('item', 'File', ''),
                ('blank', None, None),

                ('group', 'Currently Selected in Database', None),
                ('hline', None, None),
                ('item', 'Title', ''),
                ('item', 'Artist', ''),
                ('item', 'Album', ''),
                ('item', 'Track', ''),
                ('item', 'Genre', ''),
                ('item', 'Date', ''),
                ('item', 'Time', ''),
                ('item', 'File', ''),
                ('blank', None, None),

                ('group', 'MPD Statistics', None),
                ('hline', None, None),
                ('item', 'NumberofSongs', ''),
                ('item', 'NumberofArtists', ''),
                ('item', 'NumberofAlbums', ''),
                ('item', 'Uptime', ''),
                ('item', 'Playtime', ''),
                ('item', 'DBPlaytime', ''),
                ('item', 'DBUpdateTime', ''),
                ('blank', None, None),
                ]
        self.lines_d = None
        self._song_key_list = ['Title', 'Artist', 'Album', 'Track', 'Genre', 'Date', 'Time', 'File']
        self._stats_key_list = [\
                'Songs', 'Artists', 'Albums', 'Uptime', 'Playtime', 'DB_Playtime', 'DB_Update']

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()

    def round2(self):
        # get song info.

        # cp = currently playing
        # siq = selected in queue
        # sid = selected in database

        # on success, _cp and _siq are nonempty dicts.
        # on failure, _cp and _siq are empty dicts.
        self._cp = self.currentsong
        try:
            self._siq = self.board.get('queue-selected') or {}
        except (mpd.CommandError, IndexError):
            self._siq = {}
        try:
            uri = self.board.get('database-selected')
            if uri and uri != self._dburi and not self.main.idle:
                self._sid = self.mpc.listallinfo(uri)[0]
        except (mpd.CommandError, IndexError):
            self._sid = {}

        # setup sub lists
        cp_list = [('item', k, self._cp.get(k.lower()) or '') for k in self._song_key_list]
        siq_list = [('item', k, self._siq.get(k.lower()) or '') for k in self._song_key_list]
        sid_list = [('item', k, self._sid.get(k.lower()) or '') for k in self._song_key_list]
        stats_list = [('item', k, self.stats.get(k.lower()) or '') for k in self._stats_key_list]

        # convert list (multi-tags)  to str
        for l in (cp_list, siq_list, sid_list):
            for i in range(6):
                l[i] = (l[i][0], l[i][1], \
                        isinstance(l[i][2], str) and l[i][2] or ', '.join(l[i][2]))

        # format time
        for l in (cp_list, siq_list, sid_list):
            l[6] = (l[6][0], l[6][1], format_time(l[6][2]))
        for i in range(3, 6):
            stats_list[i] = (stats_list[i][0], stats_list[i][1], format_time(stats_list[i][2]))
        stats_list[6] = (stats_list[6][0], stats_list[6][1], \
                time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(int(stats_list[6][2]))))

        # merge into main list
        self.lines[2:10] = cp_list
        self.lines[13:21] = siq_list
        self.lines[24:32] = sid_list
        self.lines[35:42] = stats_list

        # set up options display
        self.lines_d = self.lines[:]
        # breakup file paths
        for k in (31, 20, 9):
            self.lines_d[k:k+1] = [('item', '', '/' + i) for i in self.lines[k][2].split('/')]
            self.lines_d[k] = ('item', 'File', self.lines_d[k][2][1:])

        self.num = len(self.lines_d)

    def update_win(self):
        self.win.erase()
        for i in range(self.beg, min(self.beg + self.height, self.num)):
            line = self.lines_d[i]
            if line[0] == 'group':
                self.win.insstr(i - self.beg, 6, line[1], curses.A_BOLD)
            elif line[0] == 'hline':
                self.win.attron(curses.A_BOLD)
                self.win.hline(i - self.beg, 3, '-', self.width - 6)
                self.win.attroff(curses.A_BOLD)
            elif line[0] == 'item':
                self.win.insstr(i - self.beg, 0, line[1].rjust(20) + ' : ' + line[2])
            elif line[0] == 'blank':
                pass
        self.win.noutrefresh()

class ArtistAlbumPane(CursedPane):
    '''List artists/albums in database.'''

    def __init__(self, win, main):
        CursedPane.__init__(self, win, main)

        # current displayed dir
        self._type = 'artist'
        self._artist = None
        self._album = None
        self.items = self.build()

    def build(self):
        '''Build view using self._type, self._artist and self._album.

        A view is rebuilt when self._type changes.'''

        if self._type == 'artist':
            view = self.mpc.list('artist')
        elif self._type == 'album':
            view = self._artist and self.mpc.list('album', self._artist) or []
        elif self._type == 'song':
            view = self._album and self.mpc.find('album', self._album) or []

        self.num = len(view)
        self.beg = 0
        self.sel = 0
        return view

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()
        elif c == ord('H'):
            self.select_top()
        elif c == ord('M'):
            self.select_middle()
        elif c == ord('L'):
            self.select_bottom()
        elif c == ord('g'):
            self.to_first()
        elif c == ord('G'):
            self.to_last()
        elif c == ord('\''):
            if self._type == 'artist':
                pass
            elif self._type == 'album':
                self._type = 'artist'
                self.items = self.build()
                for i in range(len(self.items)):
                    if self.items[i] == self._artist:
                        self.locate(i)
                        break
            elif self._type == 'song':
                self._type = 'album'
                self.items = self.build()
                for i in range(len(self.items)):
                    if self.items[i] == self._album:
                        self.locate(i)
                        break
        elif c == ord('"'):
            self._type = 'artist'
            self.items = self.build()
        elif c == ord('\n'):
            item = self.items[self.sel]
            if self._type == 'artist':
                self._artist = item
                self._type = 'album'
                self.items = self.build()
            elif self._type == 'album':
                self._album = item
                self._type = 'song'
                self.items = self.build()
            elif self._type == 'song':
                uri = item['file']
                songs = self.mpc.playlistfind('file', uri)
                if songs:
                    self.mpc.playid(songs[0]['id'])
                else:
                    self.mpc.add(uri)
                    song = self.mpc.playlistfind('file', uri)[0]
                    self.mpc.playid(song['id'])
        elif c == ord('a'):
            item = self.items[self.sel]
            if self._type == 'artist':
                self.mpc.findadd('artist', item)
            elif self._type == 'album':
                self.mpc.findadd('album', item)
            elif self._type == 'song':
                self.mpc.add(item['file'])
        elif c in [ord('/'), ord('?'), ord('n'), ord('N')]:
            self.search('Artist-Album', c)
        elif c == ord(';'):
            # tell QUEUE we want to locate a song
            if self._type == 'song':
                item = self.items[self.sel]
                self.board['queue-locate'] = item.get('file')
            else:
                self.board['msg'] = 'No song selected'

    def round2(self):
        if ('database-updated' in self.board):
            self._type = 'artist'
            self.items = self.build()

    def update_win(self):
        self.win.erase()
        for i in range(self.beg, min(self.beg + self.height, self.num)):
            item = self.items[i]

            if self._type in ['artist', 'album']:
                val = item
            elif self._type == 'song':
                val = get_tag('title', item) or os.path.basename(item.get('file'))

            if i == self.sel:
                self.win.attron(curses.A_REVERSE)
            if self._type == 'artist':
                self.win.attron(curses.color_pair(1) | curses.A_BOLD)
            elif self._type == 'album':
                self.win.attron(curses.color_pair(2) | curses.A_BOLD)
            self.win.hline(i - self.beg, 0, ' ', self.width)
            self.win.insstr(i - self.beg, 0, val)
            if self._type == 'artist':
                self.win.attroff(curses.color_pair(1) | curses.A_BOLD)
            elif self._type == 'album':
                self.win.attroff(curses.color_pair(2) | curses.A_BOLD)
            if i == self.sel:
                self.win.attroff(curses.A_REVERSE)
        self.win.noutrefresh()

class SearchPane(CursedPane):
    '''Search in the database.'''

    def __init__(self, win, main):
        CursedPane.__init__(self, win, main)

        self.items = []

    def build(self, kw):
        '''Build view using search keywords.'''

        try:
            name, value = kw.split(':', 1)
            view = self.mpc.find(name, value) or []
            if not view:
                self.board['msg'] = 'Nothing found :('
        except:
            view = []
            self.board['msg'] = 'Syntax is <tag_name>:<tag_value>'

        self.num = len(view)
        self.beg = 0
        self.sel = 0
        return view

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()
        elif c == ord('H'):
            self.select_top()
        elif c == ord('M'):
            self.select_middle()
        elif c == ord('L'):
            self.select_bottom()
        elif c == ord('g'):
            self.to_first()
        elif c == ord('G'):
            self.to_last()
        elif c == ord('B'):
            self.items = self.build(self.main.e.getstr('Database Search'))
        elif c == ord('\n'):
            item = self.items[self.sel]
            uri = item['file']
            songs = self.mpc.playlistfind('file', uri)
            if songs:
                self.mpc.playid(songs[0]['id'])
            else:
                self.mpc.add(uri)
                song = self.mpc.playlistfind('file', uri)[0]
                self.mpc.playid(song['id'])
        elif c == ord('a'):
            item = self.items[self.sel]
            self.mpc.add(item['file'])
        elif c in [ord('/'), ord('?'), ord('n'), ord('N')]:
            self.search('Search', c)
        elif c == ord(';'):
            # tell QUEUE we want to locate a song
            if self.sel < self.num:
                item = self.items[self.sel]
                self.board['queue-locate'] = item.get('file')
            else:
                self.board['msg'] = 'No song selected'

    def update_win(self):
        self.win.erase()
        for i in range(self.beg, min(self.beg + self.height, self.num)):
            item = self.items[i]

            val = get_tag('title', item) or os.path.basename(item.get('file'))

            if i == self.sel:
                self.win.attron(curses.A_REVERSE)
            self.win.hline(i - self.beg, 0, ' ', self.width)
            self.win.insstr(i - self.beg, 0, val)
            if i == self.sel:
                self.win.attroff(curses.A_REVERSE)
        self.win.noutrefresh()

class OutputPane(CursedPane):
    '''Output pane.'''

    def __init__(self, win, main):
        CursedPane.__init__(self, win, main)
        self.outputs = []

    def update_data(self):
        CursedPane.update_data(self)

        self.outputs = self.mpc.outputs()
        self.num = len(self.outputs)
        self.beg = self.clamp(self.beg)
        self.sel = self.clamp(self.sel)

    def round1(self, c):
        if c == ord('j'):
            self.line_down()
        elif c == ord('k'):
            self.line_up()
        elif c == ord('f'):
            self.page_down()
        elif c == ord('b'):
            self.page_up()
        elif c == ord('t'):
            output = self.outputs[self.sel]
            output_id = int(output['outputid'])
            output_enabled = int(output['outputenabled'])
            if output_enabled:
                self.mpc.disableoutput(output_id)
                self.outputs[self.sel]['outputenabled'] = '0'
            else:
                self.mpc.enableoutput(output_id)
                self.outputs[self.sel]['outputenabled'] = '1'

    def update_win(self):
        self.win.erase()

        for i in range(self.beg, min(self.beg + self.height, self.num)):
            item = self.outputs[i]

            if i == self.sel:
                self.win.attron(curses.A_REVERSE)

            enabled_str = '[{}]'.format('o' if int(item['outputenabled']) else 'x')
            name_str = item['outputname']
            item_str = '{} {}'.format(enabled_str, name_str)

            self.win.hline(i - self.beg, 0, ' ', self.width)
            self.win.insstr(i - self.beg, 0, item_str)

            if i == self.sel:
                self.win.attroff(curses.A_REVERSE)

        self.win.noutrefresh()




#!/usr/bin/env python3
# -*- Mode: python; coding: utf-8; tab-width: 8; indent-tabs-mode: t; -*-
#
# Copyright 2007 Sevenever
# Copyright (C) 2007 Sevenever
# Modified: (2011) Cyker Way
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.

import sys
import locale
import codecs
from urllib.request import urlopen
from urllib.request import Request
import random
from xml.dom.minidom import parse, parseString

def CodeFunc(Id, data):
    length = len(data)

    tmp2=0
    tmp3=0

    tmp1 = (Id & 0x0000FF00) >> 8							#右移8位后为0x0000015F
                                                            #tmp1 0x0000005F
    if ( (Id & 0x00FF0000) == 0 ):
        tmp3 = 0x000000FF & ~tmp1							#CL 0x000000E7
    else:
        tmp3 = 0x000000FF & ((Id & 0x00FF0000) >> 16)		#右移16位后为0x00000001

    tmp3 = tmp3 | ((0x000000FF & Id) << 8)					#tmp3 0x00001801
    tmp3 = tmp3 << 8										#tmp3 0x00180100
    tmp3 = tmp3 | (0x000000FF & tmp1)						#tmp3 0x0018015F
    tmp3 = tmp3 << 8										#tmp3 0x18015F00
    if ( (Id & 0xFF000000) == 0 ) :
        tmp3 = tmp3 | (0x000000FF & (~Id))					#tmp3 0x18015FE7
    else :
        tmp3 = tmp3 | (0x000000FF & (Id >> 24))			#右移24位后为0x00000000

    #tmp3	18015FE7

    i=length-1
    while(i >= 0):
        char = (data[i])
        if char >= 0x80:
            char = char - 0x100
        tmp1 = (char + tmp2) & 0x00000000FFFFFFFF
        tmp2 = (tmp2 << (i%2 + 4)) & 0x00000000FFFFFFFF
        tmp2 = (tmp1 + tmp2) & 0x00000000FFFFFFFF
        #tmp2 = ((data[i])) + tmp2 + ((tmp2 << (i%2 + 4)) & 0x00000000FFFFFFFF)
        i -= 1

    #tmp2 88203cc2
    i=0
    tmp1=0
    while(i<=length-1):
        char = (data[i])
        if char >= 128:
            char = char - 256
        tmp7 = (char + tmp1) & 0x00000000FFFFFFFF
        tmp1 = (tmp1 << (i%2 + 3)) & 0x00000000FFFFFFFF
        tmp1 = (tmp1 + tmp7) & 0x00000000FFFFFFFF
        #tmp1 = ((data[i])) + tmp1 + ((tmp1 << (i%2 + 3)) & 0x00000000FFFFFFFF)
        i += 1

    #EBX 5CC0B3BA

    #EDX = EBX | Id
    #EBX = EBX | tmp3
    tmp1 = (((((tmp2 ^ tmp3) & 0x00000000FFFFFFFF) + (tmp1 | Id)) & 0x00000000FFFFFFFF) * (tmp1 | tmp3)) & 0x00000000FFFFFFFF
    tmp1 = (tmp1 * (tmp2 ^ Id)) & 0x00000000FFFFFFFF

    if tmp1 > 0x80000000:
        tmp1 = tmp1 - 0x100000000
    return tmp1

def EncodeArtTit(str):
    rtn = ''
    str = str.encode('UTF-16')[2:]
    for i in range(len(str)):
        rtn += '%02x' % (str[i])

    return rtn

def fetch_lyrics(artist, title):
    # fixed headers
    headers =  {'User-agent' : 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)'}

    try:
        url = 'http://ttlrcct2.qianqian.com/dll/lyricsvr.dll?sh?Artist={}&Title={}&Flags=0'.format(
                EncodeArtTit(artist.replace(' ','').lower()),
                EncodeArtTit(title.replace(' ','').lower()),
                )
        req = Request(url, None, headers)
        handle = urlopen(req)
    except IOError:
        return '[00:00.00]Lyrics fetching failed.'
    else:
        dom = parseString(handle.read())
        list = dom.getElementsByTagName('lrc')
        li = []
        for node in list:
            li.append((node.getAttribute('id'),node.getAttribute('artist'),node.getAttribute('title')))

        # only use the first element
        if not li:
            return '[00:00.00]Lyrics not found.'
        li = li[0]

        try:
            url = 'http://ttlrcct2.qianqian.com/dll/lyricsvr.dll?dl?Id=%d&Code=%d&uid=01&mac=%012x' % (
                    int(li[0]),
                    CodeFunc(int(li[0]), (li[1] + li[2]).encode('UTF-8')),
                    random.randint(0,0xFFFFFFFFFFFF)
                    )
            req = Request(url, None, headers)
            handle = urlopen(req)
            # and open it to return a handle on the url
        except IOError:
            return '[00:00.00]Lyrics not found.'
        else:
            return handle.read().decode()

if __name__ == '__main__':
    print(fetch_lyrics(sys.argv[1], sys.argv[2]))




#!/usr/bin/env python3

'''Util module.'''

import sys

def printerr(s):
    sys.stderr.write(s + '\n')

def format_time(tm):
    '''Convert time: <seconds> -> <hh:mm:ss>.'''

    if tm.isdigit():
        tm = int(tm)
        h, m, s = tm // 3600, (tm // 60) % 60, tm % 60
        return '{}:'.format(h) * bool(h > 0) + '{:02d}:{:02d}'.format(m, s)
    else:
        return ''

def get_tag(tagname, item):
    tag = item.get(tagname)
    if isinstance(tag, str):
        return tag
    elif isinstance(tag, list):
        return ', '.join(tag)
    else:
        return None




#!/usr/bin/env python3

'''Lyrics module.'''

import re

class LRCParseError(Exception):
    pass

_tag = re.compile(r'\[([a-z][a-z]):(.*)\]')
_tm = re.compile(r'\[(\d\d):(\d\d).(\d\d)\](.*)')

def parse(lrc):
    tags = {}
    tms = {}

    try:
        lines = lrc.splitlines()
        for line in lines:
            line = line.strip()
            m = _tag.match(line)
            if m:
                tagkey, tagval = m.groups()
                tags[tagkey] = tagval.strip()
            else:
                # handle multiple tms, like [mm:ss.xx]<mm:ss.xx>...
                matched_tms = []
                m = _tm.match(line)
                while m:
                    mm, ss, xx, line = m.groups()
                    tm = float(mm) * 60 + float(ss) + float(xx) * 0.01
                    matched_tms.append(tm)
                    m = _tm.match(line)
                for tm in matched_tms:
                    tms[tm] = line
    except:
        raise LRCParseError('Cannot Parse')
    finally:
        return tags, tms


def get_title(lrc):
    tags, tms = parse(lrc)
    return tags.get('ti')


def get_artist(lrc):
    tags, tms = parse(lrc)
    return tags.get('ar')


def get_album(lrc):
    tags, tms = parse(lrc)
    return tags.get('al')


def get_by(lrc):
    tags, tms = parse(lrc)
    return tags.get('by')


def get_text(lrc):
    tags, tms = parse(lrc)
    for tm in sorted(tms.keys()):
        yield tms.get(tm)


def compile(lrc):
    return LRC(*parse(lrc))


class LRC():
    def __init__(self, tags, tms):
        self.tags, self.tms = tags, tms
    def parse(self):
        return self.tags, self.tms
    def get_title(self):
        return self.tags.get('ti')
    def get_artist(self):
        return self.tags.get('ar')
    def get_album(self):
        return self.tags.get('al')
    def get_by(self):
        return self.tags.get('by')
    def get_text(self):
        for tm in sorted(self.tms.keys()):
            yield self.tms.get(tm)

def test():
    with open('./test.lrc', 'rt') as f:
        s = f.read()

    print('module method (no compile)')
    lrc = compile(s)
    print('artist = ', lrc.get_artist())
    print('title = ', lrc.get_title())
    print('album = ', lrc.get_album())
    print('by = ', lrc.get_by())
    texts = lrc.get_text()
    for text in texts:
        print(text)

    print('class method (compile)')
    print('artist = ', get_artist(s))
    print('title = ', get_title(s))
    print('album = ', get_album(s))
    print('by = ', get_by(s))
    texts = get_text(s)
    for text in texts:
        print(text)


if __name__ == '__main__':
    test()



#!/usr/bin/env python3

'''
setuptools based setup module;

see <https://packaging.python.org/en/latest/distributing.html>;
'''

from os import path
from setuptools import find_packages
from setuptools import setup

here = path.abspath(path.dirname(__file__))

##  get long description from readme file;
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    ##  ========================================================================
    ##  required for pypi upload;
    ##  ========================================================================

    ##  project name;
    ##
    ##  this determines how users install this project:
    ##
    ##      pip install sampleproject
    ##
    ##  and where this project lives on pypi:
    ##
    ##  <https://pypi.org/project/sampleproject/>
    ##
    ##  this name is registered for you the first time you publish this package;
    ##
    ##  name specification:
    ##
    ##  <https://packaging.python.org/specifications/core-metadata/#name>
    ##
    name='ncmpy',

    ##  project version;
    ##
    ##  version specification (pep 440):
    ##
    ##  <https://www.python.org/dev/peps/pep-0440/>;
    ##
    ##  single-sourcing techniques:
    ##
    ##  <https://packaging.python.org/en/latest/single_source_version.html>
    ##
    version='1.5.2',

    ##  project homepage;
    ##
    ##  this arg corresponds to "home-page" metadata field:
    ##
    ##  <https://packaging.python.org/specifications/core-metadata/#home-page-optional>
    ##
    url='https://github.com/cykerway/ncmpy',

    ##  author name;
    author='Cyker Way',

    ##  author email address;
    author_email='cykerway@example.com',

    ##  packages;
    ##
    ##  you can provide a list of packages manually or use `find_packages()`;
    ##
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    ##  ========================================================================
    ##  optional for pypi upload;
    ##  ========================================================================

    ##  a one-line description;
    ##
    ##  this arg corresponds to "summary" metadata field:
    ##
    ##  <https://packaging.python.org/specifications/core-metadata/#summary>
    ##
    description='a curses mpd client;',

    ##  a longer description shown on project homepage on pypi;
    ##
    ##  this is often the same as the readme;
    ##
    ##  this arg corresponds to "description" metadata field:
    ##
    ##  <https://packaging.python.org/specifications/core-metadata/#description-optional>
    ##
    long_description=long_description,

    ##  longer description content type;
    ##
    ##  valid values are: `text/plain`, `text/x-rst`, `text/markdown`;
    ##
    ##  this arg corresponds to "description-content-type" metadata field:
    ##
    ##  <https://packaging.python.org/specifications/core-metadata/#description-content-type-optional>
    ##
    long_description_content_type='text/markdown',

    ##  classifiers categorizing this project;
    ##
    ##  see <https://pypi.org/classifiers/>;
    ##
    classifiers=[
        ##  development status;
#        'Development Status :: 3 - Alpha',
#        'Development Status :: 4 - Beta',
        'Development Status :: 5 - Production/Stable',

        ##  intended audience;
#        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',

        ##  topic;
#        'Topic :: Desktop Environment',
#        'Topic :: Games/Entertainment',
        'Topic :: Multimedia',
#        'Topic :: Office/Business',
#        'Topic :: Scientific/Engineering',
#        'Topic :: Software Development',
#        'Topic :: System',

        ##  license;
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
#        'License :: OSI Approved :: BSD License',
#        'License :: OSI Approved :: MIT License',

        ##  supported python versions;
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    ##  project keywords;
    ##
    ##  these keywords will appear on the project page;
    ##
    keywords='mpd music player',

    ##  package data;
    ##
    ##  this is a dict mapping package names to a list of relative path names
    ##  (or glob patterns) that should be copied into the package when
    ##  installed; the path names are interpreted relative to the package dir;
    ##
    package_data={
#        'sample': ['*.bin'],
    },

    ##  additional data files;
    ##
    ##  this is a sequence of `(dir, files)` pairs; each `(dir, files)` pair
    ##  specifies the install dir and the files to install there; if `dir` is a
    ##  relative path, it is relative to the install prefix (`sys.prefix` or
    ##  `sys.exec_prefix`); each file in `files` is interpreted relative to the
    ##  `setup.py` script;
    ##
    ##  see <https://docs.python.org/3/distutils/setupscript.html#installing-additional-files>;
    ##
    data_files=[
#        ('data_files', ['data/data0.bin', 'data/data1.bin']),
    ],

    ##  package dependencies;
    ##
    ##  this is a list of packages that this project depends on; these packages
    ##  will be installed by pip when this project is installed;
    ##
    install_requires=[
        'python-mpd2',
    ],

    ##  extra package dependencies;
    ##
    ##  this is a dict mapping extras (optional features of this project) to a
    ##  list of packages that those extras depend on;
    ##
    ##  users will be able to install these using the extras syntax:
    ##
    ##      pip install sampleproject[dev]
    ##
    ##  see <https://setuptools.readthedocs.io/en/latest/setuptools.html#declaring-extras-optional-features-with-their-own-dependencies>
    ##
    extras_require={
#        'dev': ['check-manifest'],
#        'test': ['coverage'],
    },

    ##  to create executable scripts, use entry points:
    ##
    ##  <https://setuptools.readthedocs.io/en/latest/setuptools.html#automatic-script-creation>
    ##
    ##  for example, the following would provide a console script `sample-cli`
    ##  which executes the `main` function in package `sample.cli`, and a gui
    ##  script `sample-gui` which executes the `main` function in package
    ##  `sample.gui`;
    entry_points={
        'console_scripts': [
            'ncmpy=ncmpy.__main__:main',
        ],
#        'gui_scripts': [
#            'sample-gui=sample.gui:main',
#        ],
    },

    ##  additional urls that are relevant to this project;
    ##
    ##  examples include: where the package tracks issues, where the source is
    ##  hosted, where to say thanks to the package maintainers, and where to
    ##  support the project financially; the keys are used to render the link
    ##  texts on pypi;
    ##
    ##  this arg corresponds to "project-url" metadata fields:
    ##
    ##  <https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use>
    ##
    project_urls={
        'Bug Reports': 'https://github.com/cykerway/ncmpy/issues',
#        'Funding': 'https://donate.pypi.org',
#        'Say Thanks!': 'http://saythanks.io/to/example',
        'Source': 'https://github.com/cykerway/ncmpy/',
    },
)



#!/usr/bin/env python3
#
# ncmpy - A curses-based MPD client written in Python.
#
# Copyright (C) 2011-2015 Cyker Way
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''
main module;
'''

import curses
import locale
import mpd
import os
import select
import signal
import sys
import time

from ncmpy.config import *
from ncmpy.pane import *
from ncmpy.util import *

class Ncmpy():
    '''Main controller.'''

    def _init_curses(self):
        self.stdscr = curses.initscr()
        curses.start_color()
        curses.use_default_colors()
        curses.noecho()
        curses.cbreak()
        curses.curs_set(0)
        curses.init_pair(1, curses.COLOR_BLUE, -1)
        curses.init_pair(2, curses.COLOR_GREEN, -1)
        curses.init_pair(3, curses.COLOR_YELLOW, -1)
        self.stdscr.keypad(1)
        self.stdscr.leaveok(1)
        # Force a refresh. Otherwise strange things happen on first key press.
        self.stdscr.refresh()

    def _init_mpd(self, host, port):
        self.mpc = mpd.MPDClient()
        self.mpc.connect(host, port)

    def _init_conf(self):
        '''Initialize internal configurations.'''

        # Main configuration.
        self.height, self.width = self.stdscr.getmaxyx()
        self.tpanename = 'Queue'
        self.loop = False
        self.idle = False
        self.seek = False
        self.sync = True
        self.elapsed = 0
        self.total = 0
        self.search = ''
        self.search_di = 0
        self.pending = []

        # No sync keys. These keys don't modify MPD server state and can be handled locally.
        # Therefore we don't sync with MPD server when handling these keys.
        self.nsks = [
                ord('j'), ord('k'), ord('f'), ord('b'),
                ord('H'), ord('M'), ord('L'), ord('g'), ord('G'),
                ord('J'), ord('K'),
                curses.KEY_UP, curses.KEY_DOWN, curses.KEY_LEFT, curses.KEY_RIGHT,
                ]

        # User input.
        self.c = None

    def _init_data(self):
        self.status = self.mpc.status()
        self.stats = self.mpc.stats()
        self.currentsong = self.mpc.currentsong()

    def _init_board(self):
        self.board = {}

    def _init_panes(self):
        '''Initialize panes.'''

        self.m = MenuPane(self.stdscr.subwin(1, self.width, 0, 0), self)
        self.t = LinePane(self.stdscr.subwin(1, self.width, 1, 0), self)

        self.p = ProgressPane(self.stdscr.subwin(1, self.width, self.height - 2, 0), self)
        self.s = StatusPane(self.stdscr.subwin(1, self.width, self.height - 1, 0), self)
        self.e = MessagePane(self.stdscr.subwin(1, self.width, self.height - 1, 0), self)

        self.h = HelpPane(curses.newwin(self.height - 4, self.width, 2, 0), self)
        self.q = QueuePane(curses.newwin(self.height - 4, self.width, 2, 0), self)
        self.d = DatabasePane(curses.newwin(self.height - 4, self.width, 2, 0), self)
        self.l = LyricsPane(curses.newwin(self.height - 4, self.width, 2, 0), self)
        self.a = ArtistAlbumPane(curses.newwin(self.height - 4, self.width, 2, 0), self)
        self.r = SearchPane(curses.newwin(self.height - 4, self.width, 2, 0), self)
        self.i = InfoPane(curses.newwin(self.height - 4, self.width, 2, 0), self)
        self.o = OutputPane(curses.newwin(self.height - 4, self.width, 2, 0), self)

        # Pane dict.
        self.pdict = {
                'Menu' : self.m,
                'Title' : self.t,
                'Progress' : self.p,
                'Status' : self.s,
                'Message' : self.e,

                'Help' : self.h,
                'Queue' : self.q,
                'Database' : self.d,
                'Lyrics' : self.l,
                'Artist-Album' : self.a,
                'Search' : self.r,
                'Info' : self.i,
                'Output' : self.o,
                }
        # Pane list.
        self.plist = self.pdict.values()

        # Bar pane dict.
        self.bpdict = {
                'Menu' : self.m,
                'Title' : self.t,
                'Progress' : self.p,
                'Status' : self.s,
                'Message' : self.e,
                }
        # Bar pane list.
        self.bplist = self.bpdict.values()

    def __enter__(self):
        self._init_curses()
        self._init_mpd(conf.mpd_host, conf.mpd_port)
        self._init_conf()
        self._init_data()
        self._init_board()
        self._init_panes()

        # start lyrics daemon thread
        self.l.daemon = True
        self.l.start()

        # initial update
        self.process('timeout')

        return self

    def __exit__(self, type, value, traceback):

        curses.endwin()

    def update_data(self):
        # Update data from MPD.
        self.status = self.mpc.status()
        self.stats = self.mpc.stats()
        self.currentsong = self.mpc.currentsong()

        # Update panes data.
        for pane in self.plist:
            pane.update_data()

    def round1(self, c):
        # Seeking.
        if c in (curses.KEY_LEFT, curses.KEY_RIGHT, curses.KEY_DOWN, curses.KEY_UP):
            if self.status['state'] in ['play', 'pause']:
                if not self.seek:
                    self.seek = True
                    self.elapsed, self.total = [int(i) for i in self.status['time'].split(':')]
                if c == curses.KEY_LEFT:
                    self.elapsed = max(self.elapsed - 1, 0)
                elif c == curses.KEY_RIGHT:
                    self.elapsed = min(self.elapsed + 1, self.total)
                elif c == curses.KEY_DOWN:
                    self.elapsed = max(self.elapsed - max(self.total // 100, 1), 0)
                elif c == curses.KEY_UP:
                    self.elapsed = min(self.elapsed + max(self.total // 100, 1), self.total)
                self.status['time'] = '{}:{}'.format(self.elapsed, self.total)
        else:
            if self.seek:
                if self.status['state'] in ['play', 'pause']:
                    self.mpc.seekid(self.status['songid'], self.elapsed)
                self.seek = False
                self.status['time'] = '{}:{}'.format(self.elapsed, self.total)

        # Volume control.
        if c == ord('9'):
            new_vol = max(int(self.status['volume']) - 1, 0)
            try:
                self.mpc.setvol(new_vol)
                self.status['volume'] = str(new_vol)
            except mpd.CommandError as e:
                self.board['msg'] = str(e)
        elif c == ord('0'):
            new_vol = min(int(self.status['volume']) + 1, 100)
            try:
                self.mpc.setvol(new_vol)
                self.status['volume'] = str(new_vol)
            except mpd.CommandError as e:
                self.board['msg'] = str(e)

        # Playback control.
        elif c == ord(' '):
            self.mpc.pause()
        elif c == ord('s'):
            self.mpc.stop()
        elif c == ord('<'):
            self.mpc.previous()
        elif c == ord('>'):
            self.mpc.next()

        # Mode control.
        elif c == ord('u'):
            self.status['consume'] = 1 - int(self.status['consume'])
            self.mpc.consume(self.status['consume'])
        elif c == ord('i'):
            self.status['random'] = 1 - int(self.status['random'])
            self.mpc.random(self.status['random'])
        elif c == ord('o'):
            self.status['repeat'] = 1 - int(self.status['repeat'])
            self.mpc.repeat(self.status['repeat'])
        elif c == ord('p'):
            self.status['single'] = 1 - int(self.status['single'])
            self.mpc.single(self.status['single'])

        # Playlist save/load.
        elif c == ord('S'):
            name = self.e.getstr('Save')
            try:
                self.mpc.save(name)
            except mpd.CommandError as e:
                self.board['msg'] = str(e).rsplit('} ')[1]
            else:
                self.board['msg'] = 'Playlist {} saved'.format(name)
                self.board['playlist'] = 'saved'
        elif c == ord('O'):
            name = self.e.getstr('Load')
            try:
                self.mpc.load(name)
            except mpd.CommandError as e:
                self.board['msg'] = str(e).rsplit('} ')[1]
            else:
                self.board['msg'] = 'Playlist {} loaded'.format(name)

        # Basic search.
        elif c in [ord('/'), ord('?')]:
            search = self.e.getstr('Find')
            if search:
                self.search = search
                if c == ord('/'):
                    self.search_di = 1
                elif c == ord('?'):
                    self.search_di = -1

        # Top pane do round1 with input char.
        self.pdict[self.tpanename].round1(c)

        # Other panes do round1 without input char.
        for panename in self.pdict:
            if panename != self.tpanename:
                self.pdict[panename].round1(-1)

        # Pane switch.
        if c == curses.KEY_F1:
            self.prevtpanename = self.tpanename
            self.tpanename = 'Help'
        elif c == curses.KEY_F2:
            self.prevtpanename = self.tpanename
            self.tpanename = 'Queue'
        elif c == curses.KEY_F3:
            self.prevtpanename = self.tpanename
            self.tpanename = 'Database'
        elif c == curses.KEY_F4:
            self.prevtpanename = self.tpanename
            self.tpanename = 'Lyrics'
        elif c == curses.KEY_F5:
            self.prevtpanename = self.tpanename
            self.tpanename = 'Artist-Album'
        elif c == curses.KEY_F6:
            self.prevtpanename = self.tpanename
            self.tpanename = 'Search'
        elif c == curses.KEY_F7:
            if self.tpanename == 'Info':
                if self.prevtpanename:
                    self.tpanename, self.prevtpanename = self.prevtpanename, self.tpanename
            else:
                self.prevtpanename = self.tpanename
                self.tpanename = 'Info'
        elif c == curses.KEY_F8:
            self.prevtpanename = self.tpanename
            self.tpanename = 'Output'

    def round2(self):
        if ('database-locate' in self.board):
            self.tpanename = 'Database'

        if ('queue-locate' in self.board):
            self.tpanename = 'Queue'

        for pane in self.plist:
            pane.round2()

    def update_win(self):
        self.pdict[self.tpanename].update_win()

        for pane in self.bplist:
            pane.update_win()

        curses.doupdate()

    def enter_idle(self):
        '''
        Enter idle state. Must be called outside idle state.

        No return value.
        '''

        self.mpc.send_idle()
        self.idle = True

    def leave_idle(self):
        '''
        Leave idle state. Must be called inside idle state.

        Return Value: Events received in idle state.
        '''

        self.mpc.noidle()
        self.idle = False

        try:
            return self.mpc.fetch_idle()
        except mpd.PendingCommandError:
            # Return None if nothing received.
            return None

    def try_enter_idle(self):
        if not self.idle:
            self.enter_idle()

    def try_leave_idle(self):
        if self.idle:
            return self.leave_idle()

    def process(self, fd):
        '''Process timeout/mpd/stdin events. Called in main loop.'''

        # Get input if event is 'stdin'.
        if fd == 'stdin':
            c = self.c = self.stdscr.getch()
            if c == ord('q'):
                self.loop = False
                return
            elif c in self.nsks:
                self.sync = False
            else:
                self.sync = True
        else:
            c = self.c = -1
            self.sync = True

        self.board.clear()

        if self.sync:
            events = self.try_leave_idle()

            if events and 'database' in events:
                self.board['database-updated'] = None

            if self.pending:
                self.mpc.command_list_ok_begin()
                for task in self.pending:
                    exec('self.mpc.' + task)
                self.mpc.command_list_end()
                self.pending = []

            self.update_data()

        self.round1(c)
        self.round2()
        self.update_win()

        if fd == 'stdin':
            curses.flushinp()
        else:
            self.try_enter_idle()

    def resize_win(self):
        '''Reset display. Called in SIGWINCH handler.'''

        curses.endwin()
        self.stdscr.refresh()
        self.height, self.width = self.stdscr.getmaxyx()

        for pane in self.plist:
            pane.resize_win()

    def main_loop(self):
        '''Main loop.'''

        poll = select.poll()
        poll.register(self.mpc.fileno(), select.POLLIN)
        poll.register(sys.stdin.fileno(), select.POLLIN)

        self.loop = True
        while self.loop:
            try:
                responses = poll.poll(200)
                if not responses:
                    self.process('timeout')
                else:
                    for fd, event in responses:
                        if fd == self.mpc.fileno() and event & select.POLLIN:
                            self.process('mpd')
                        elif fd == sys.stdin.fileno() and event & select.POLLIN:
                            self.process('stdin')
            except select.error as e:
                # Ignore poll interruption.
                pass

    def handler(self, signum, frame):
        '''Signal handler.'''

        if signum == signal.SIGWINCH:
            # Resize window.
            self.resize_win()
            # Consume KEY_RESIZE and update.
            self.process('stdin')

def main():

    '''
    main function;
    '''

    try:
        locale.setlocale(locale.LC_ALL,'')

        if not os.path.isdir(conf.lyrics_dir):
            os.makedirs(conf.lyrics_dir)

        with Ncmpy() as ncmpy:
            signal.signal(signal.SIGWINCH, ncmpy.handler)
            ncmpy.main_loop()
    finally:
        curses.endwin()

if __name__ == '__main__':
    main()

